<?php
/**
 * Billing Overview main controller
 * 
 * @package blesta
 * @subpackage blesta.plugins.billing_overview
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class AdminMain extends BillingOverviewController {
	
	/**
	 * @var array A list of graph line hex colors
	 */
	private $line_colors = array("#0075b2", "#6ab31d", "#444a42", "#a80000");
	
	/**
	 * Pre action
	 */
	public function preAction() {
		parent::preAction();
		
		// Load settings
		$this->uses(array("BillingOverview.BillingOverviewSettings"));
		
		// Load currency helper
		$this->helpers(array("CurrencyFormat", "Form", "Html"));
		
		Language::loadLang("admin_main", null, PLUGINDIR . "billing_overview" . DS . "language" . DS);
	}
	
	/**
	 * Get graph date ranges
	 */
	private function getDateRanges() {
		// Set graph date ranges
		return array(
			7 => "7 " . Language::_("AdminMain.date_range.days", true),
			30 => "30 " . Language::_("AdminMain.date_range.days", true)
		);
	}

	/**
	 * Renders the billing overview widget
	 */
	public function index() {
		// Only available via AJAX
		if (!$this->isAjax())
			$this->redirect($this->base_uri . "billing/");
		
		// Set the overview content
		$this->set("content", $this->partial("admin_main_overview", $this->overview(false)));
		
		return $this->renderAjaxWidgetIfAsync(isset($this->get[0]) ? false : null);
	}
	
	/**
	 * Settings
	 */
	public function settings() {
		// Only available via AJAX
		if (!$this->isAjax())
			$this->redirect($this->base_uri . "billing/");
		
		// Get all overview settings
		$settings = array();
		$overview_settings = $this->BillingOverviewSettings->getSettings($this->Session->read("blesta_staff_id"), $this->company_id);
		
		foreach ($overview_settings as $setting)
			$settings[$setting->key] = $setting->value;
		
		$this->set("vars", $settings);
		$this->set("date_ranges", $this->getDateRanges());
		
		return $this->renderAjaxWidgetIfAsync(false);
	}
	
	/**
	 * Update settings
	 */
	public function update() {
		// Set default value for each unset checkbox
		if (!isset($this->post['revenue_today']))
			$this->post['revenue_today'] = 0;
		if (!isset($this->post['revenue_month']))
			$this->post['revenue_month'] = 0;
		if (!isset($this->post['invoiced_today']))
			$this->post['invoiced_today'] = 0;
		if (!isset($this->post['invoiced_month']))
			$this->post['invoiced_month'] = 0;
		if (!isset($this->post['balance_outstanding']))
			$this->post['balance_outstanding'] = 0;
		if (!isset($this->post['balance_overdue']))
			$this->post['balance_overdue'] = 0;
		if (!isset($this->post['scheduled_cancelation']))
			$this->post['scheduled_cancelation'] = 0;
		if (!isset($this->post['services_active']))
			$this->post['services_active'] = 0;
		if (!isset($this->post['services_added_today']))
			$this->post['services_added_today'] = 0;
		if (!isset($this->post['services_canceled_today']))
			$this->post['services_canceled_today'] = 0;
		if (!isset($this->post['graph_revenue']))
			$this->post['graph_revenue'] = 0;
		if (!isset($this->post['graph_invoiced']))
			$this->post['graph_invoiced'] = 0;
		if (!isset($this->post['show_legend']))
			$this->post['show_legend'] = 0;
		
		// Set each setting into indexed array for adding
		$settings = array();
		foreach ($this->post as $key=>$value)
			$settings[] = array('key'=>$key, 'value'=>$value);
		
		// Add the settings
		$this->BillingOverviewSettings->add($this->Session->read("blesta_staff_id"), $this->company_id, $settings);
		
		if (($errors = $this->BillingOverviewSettings->errors())) {
			// Error
			$this->flashMessage("error", $errors);
		}
		else {
			// Success
			$this->flashMessage("message", Language::_("AdminMain.!success.options_updated", true));
		}
		
		$this->redirect($this->base_uri . "billing/");
	}
	
	/**
	 * Retrieves the billing overview inner content
	 */
	public function overview($echo=true) {
		// Load settings, statistics, currencies
		$this->uses(array("BillingOverview.BillingOverviewStatistics", "Currencies"));
		$this->components(array("Json", "SettingsCollection"));
		
		// Set staff ID
		$staff_id = $this->Session->read("blesta_staff_id");
		
		// Set dates
		$datetime = $this->Date->format("c");
		$dates = array(
			'today_start' => $this->Date->cast($datetime, "Y-m-d 00:00:00"),
			'today_end' => $this->Date->cast($datetime, "Y-m-d 23:59:59"),
			'month_start' => $this->Date->cast($datetime, "Y-m-01 00:00:00"),
			'month_end' => $this->Date->cast($datetime, "Y-m-t 23:59:59")
		);
		
		// Set currency
		$default_currency = $this->SettingsCollection->fetchSetting(null, $this->company_id, "default_currency");
		$currency = (isset($default_currency['value']) ? $default_currency['value'] : "");
		
		if (!empty($this->post['currency']))
			$currency = $this->post['currency'];
		
		// Get the statistics to show for this user
		$overview_settings = $this->BillingOverviewSettings->getSettings($staff_id, $this->company_id);
		
		// Set default settings for this staff member if none yet exist
		if (empty($overview_settings)) {
			$this->BillingOverviewSettings->addDefault($staff_id, $this->company_id);
			$overview_settings = $this->BillingOverviewSettings->getSettings($staff_id, $this->company_id);
		}
		
		// Set which statistics to show
		$active_statistics = array();
		foreach ($overview_settings as $setting) {
			if ($setting->value == 1)
				$active_statistics[] = $setting->key;
		}
		
		// Set statistics
		$statistics = array();
		// Get each statistic's data
		foreach ($active_statistics as $statistic) {
			$value = "";
			$value_class = "";
			
			// Set statistic-specific values
			switch ($statistic) {
				case "revenue_today":
					// Get today's revenue
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $dates['today_start'], $dates['today_end']), $currency);
					$value_class = "more";
					break;
				case "revenue_month":
					// Get this month's revenue
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $dates['month_start'], $dates['month_end']), $currency);
					$value_class = "more";
					break;
				case "invoiced_today":
					// Get today's invoice total
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getAmountInvoiced($this->company_id, $currency, $dates['today_start'], $dates['today_end']), $currency);
					$value_class = "neutral";
					break;
				case "invoiced_month":
					// Get this month's invoice total
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getAmountInvoiced($this->company_id, $currency, $dates['month_start'], $dates['month_end']), $currency);
					$value_class = "neutral";
					break;
				case "balance_outstanding":
					// Get the total amount to be paid
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getOutstandingBalance($this->company_id, $currency), $currency);
					$value_class = "neutral";
					break;
				case "balance_overdue":
					// Get the total amount past due
					$value = $this->CurrencyFormat->format($this->BillingOverviewStatistics->getOverdueBalance($this->company_id, $currency), $currency);
					$value_class = "less";
					break;
				case "scheduled_cancelation":
					// Get the number of service cancelations
					$value = $this->BillingOverviewStatistics->getScheduledCancelationsCount($this->company_id);
					$value_class = "neutral";
					break;
				case "services_active":
					// Get the number of active services
					$value = $this->BillingOverviewStatistics->getActiveServicesCount($this->company_id);
					$value_class = "neutral";
					break;
				case "services_added_today":
					// Get the number of services added today
					$value = $this->BillingOverviewStatistics->getServicesAddedCount($this->company_id, $dates['today_start'], $dates['today_end']);
					$value_class = "neutral";
					break;
				case "services_canceled_today":
					// Get the number of services canceled today
					$value = $this->BillingOverviewStatistics->getServicesCanceledCount($this->company_id, $dates['today_start'], $dates['today_end']);
					$value_class = "neutral";
					break;
				default:
					// Move on, this is not a statistic setting
					continue 2;
			}
			
			$statistics[] = array(
				'class' => $statistic,
				'name' => Language::_("AdminMain.index.statistic." . $statistic, true),
				'value' => $value,
				'value_class' => $value_class
			);
		}
		
		$data = array(
			'vars' => array('currency'=>$currency),
			'currencies' => $this->Form->collapseObjectArray($this->Currencies->getAll($this->company_id), "code", "code"),
			'statistics' => $statistics,
			'graphs' => $this->getGraphs($currency, $overview_settings)
		);
		
		// Return the overview data
		if (!$echo)
			return $data;
		
		$this->outputAsJson(array('overview'=>$this->partial("admin_main_overview", $data)));
		return false;
	}
	
	/**
	 * Sets up the data for each graph
	 *
	 * @param string $currency The ISO 4217 currency code
	 * @param array $settings The plugin settings (optional)
	 * @return array A list of graph data and settings
	 */
	private function getGraphs($currency, $settings=null) {
		// Get settings if not given
		if ($settings == null)
			$settings = $this->BillingOverviewSettings->getSettings($this->Session->read("blesta_staff_id"), $this->company_id);
		
		// Get date range of graphs
		$graph_settings = array();
		foreach ($settings as $setting) {
			if ($setting->key == "date_range" || $setting->key == "show_legend")
				$graph_settings[$setting->key] = $setting->value;
		}
		
		// Get each graph in use
		$graphs = array('graphs'=>array(), 'settings'=>$graph_settings, 'line_colors'=>$this->line_colors);
	
		foreach ($settings as $setting) {
			switch ($setting->key) {
				case "graph_invoiced":
				case "graph_revenue":
					if ($setting->value == 1) {
						// Get the graph data over the set interval
						$graph = $this->getGraph($setting->key, (isset($graph_settings['date_range']) ? $graph_settings['date_range'] : 0), $currency);
						
						// Set each graph line name
						$line_names = array();
						$points = array();
						foreach ($graph as $key=>$value) {
							$line_names[] = Language::_("AdminMain.graph_line_name." . $key, true);
							$points[] = $value;
						}
						
						$graphs['graphs'][$setting->key] = array(
							'name'=>Language::_("AdminMain.graph_name." . $setting->key, true),
							'data' => array(
								'names' => $line_names,
								'points' => $this->Json->encode($points)
							)
						);
					}
					break;
			}
		}
		
		return $graphs;
	}
	
	/**
	 * Retrieves graph data over a given interval
	 *
	 * @param string $key The graph setting key
	 * @param int $days The time interval in days to retrieve data from
	 * @param string $currency The ISO 4217 currency code
	 * @return array An array of line data representing the graph
	 */
	private function getGraph($key, $days, $currency) {
		// Set values for each graph line
		$lines = array();
		$datetime = $this->Date->format("c");
		for ($i=0; $i<max(0, (int)$days); $i++) {
			// Set start/end dates
			$date = date("Y-m-d 23:59:59", strtotime($datetime . " -" . $i . " days"));
			$day_start = $this->Date->cast($date, "Y-m-d 00:00:00");
			$day_end = $this->Date->cast($date, "Y-m-d 23:59:59");
			
			// Set each graph data point
			switch ($key) {
				case "graph_revenue":
					/*
					$lines['total'][] = array(
						$day_start,
						(float)$this->CurrencyFormat->cast($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $day_start, $day_end), $currency)
					);
					*/
					$lines['credit'][] = array(
						$day_start,
						(float)$this->CurrencyFormat->cast($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $day_start, $day_end, "cc"), $currency)
					);
					$lines['ach'][] = array(
						$day_start,
						(float)$this->CurrencyFormat->cast($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $day_start, $day_end, "ach"), $currency)
					);
					$lines['other'][] = array(
						$day_start,
						(float)$this->CurrencyFormat->cast($this->BillingOverviewStatistics->getRevenue($this->company_id, $currency, $day_start, $day_end, "other"), $currency)
					);
					break;
				case "graph_invoiced":
					$lines['total'][] = array(
						$day_start,
						(float)$this->CurrencyFormat->cast($this->BillingOverviewStatistics->getAmountInvoiced($this->company_id, $currency, $day_start, $day_end), $currency)
					);
					break;
			}
		}
		
		// Omit lines that contain no amounts
		return $this->removeEmptyLines($lines, array("total", "other"));
	}
	
	/**
	 * Removes lines from a graph that contain only zero values
	 * @see AdminMain::getGraph()
	 *
	 * @param array An array of lines
	 * @param array A list of type exceptions that should not be removed even when zero (e.g. "total")
	 * @return array An updated array with line types of only zero values removed
	 */
	private function removeEmptyLines(array $lines, array $exceptions=array()) {
		foreach ($lines as $key=>$index) {
			// Skip key exceptions
			if (in_array($key, $exceptions))
				continue;
			
			foreach ($index as $value) {
				// Skip non-zero types
				if (isset($value[1]) && $value[1] != 0)
					continue 2;
			}
			
			unset($lines[$key]);
		}
		
		return $lines;
	}
}
?>